import datetime
from pydantic import (
    BaseModel,
    StrictStr
)


class AWSs3CopyOBjectResult(BaseModel):
    ETag: StrictStr
    LastModified: datetime.datetime
    ChecksumCRC32: StrictStr
    ChecksumCRC32C: StrictStr
    ChecksumSHA1: StrictStr
    ChecksumSHA256: StrictStr


from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBool
)

from typing import Literal, Optional, List
from .aws_s3_object_response import AWSs3ObjectResponse
from .aws_s3_common_prefix import AWSs3CommonPrefix


class AWSs3ListObjectsResponse(BaseModel):
    IsTruncated: StrictBool
    Contents: List[AWSs3ObjectResponse]=[]
    Name: StrictStr
    Prefix: StrictStr
    Delimiter: StrictStr
    MaxKeys: StrictInt
    CommonPrefixes: List[AWSs3CommonPrefix]=[]
    EncodingType: Optional[
        Literal['url']
    ]=None
    KeyCount: StrictInt
    ContinuationToken: StrictStr
    NextContinuationToken: StrictStr
    StartAfter: StrictStr
    RequestPayer: Optional[
        Literal['requester']
    ]=None


from .describe_cache_clusters_request import DescribeCacheClustersRequest as DescribeCacheClustersRequest
from .describe_cache_clusters_response import DescribeCacheClustersResponse as DescribeCacheClustersResponse


from pydantic import BaseModel, StrictStr
from typing import Literal
from .destination_details import DestinationDetails


class LogDeliveryConfiguration(BaseModel):
    LogType: Literal['slow-log', 'engine-log']
    DestinationType: Literal['cloudwatch-logs', 'kinesis-firehose']
    DestinationDetails: DestinationDetails
    LogFormat: Literal['text', 'json']
    Status: Literal[
        'active',
        'enabling',
        'modifying',
        'disabling',
        'error',
    ] | None = None
    Message: StrictStr | None = None


from pydantic import (
    StrictStr,
    StrictBool
)
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg,
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)
from typing import Optional


class AWSs3DeleteObjectOptions(AWSBoto3Base):
    mfa: Optional[StrictStr]=None
    version_id: Optional[StrictStr]=None
    bypass_governance_retention: Optional[StrictBool]=None
    expected_bucket_owner: Optional[StrictStr]=None

    def to_options(self):

        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                ['mfa']
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                ['mfa']
            )
        }

        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items() if key not in uppercased_options
        }

        parsed_options.update(uppercased_options)

        return parsed_options


from pydantic import (
    StrictStr
)
from typing import Literal, Optional, Dict
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)
from .aws_s3_versioning_configuration import AWSs3VersioningConfiguration


class AWSs3PutBucketVersioningOptions(AWSBoto3Base):
    checksum_algorithm: Optional[
        Literal[
            'CRC32',
            'CRC32C',
            'SHA1',
            'SHA256'
        ]
    ]=None
    mfa: Optional[StrictStr]=None
    versioning_configuration: Optional[
        AWSs3VersioningConfiguration
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None

    def to_options(self):

        options = self._filtered_options_to_dict()
        
        if versioning_configuration := self.versioning_configuration:
            options['versioning_configuration'] = versioning_configuration.to_data()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'mfa'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'mfa_delete'
                ]
            )
        }


        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items()
        }

        parsed_options.update(uppercased_options)

        return parsed_options


from abc import ABC, abstractmethod
from typing import Any, Literal


PolicyArn = dict[
    Literal["arn"],
    str,
]

Tag = dict[
    Literal['Key', 'Value'],
    str,
]

ProvidedContext = dict[
    Literal["ProviderArn", "ContextAssertion"],
    str,
]


class STSClient(ABC):

    @abstractmethod
    def assume_role(
        self,
        RoleArn: str,
        RoleSessionName: str,
        PolicyArns: list[PolicyArn] | None = None,
        Policy: str | None = None,
        DurationSeconds: int | None = None,
        Tags: list[Tag] | None = None,
        TransitiveTagKeys: list[str] | None = None,
        ExternalId: str | None = None,
        SerialNumber: str | None = None,
        TokenCode: str | None = None,
        SourceIdentity: str | None = None,
        ProvidedContexts: list[ProvidedContext] | None = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def close(self):
        pass


from pydantic import BaseModel
from .cloud_watch_logs_detail import CloudWatchLogsDetail
from .kinesis_firehose_delivery_detail import KinesisFirehoseDetail


class DestinationDetails(BaseModel):
    CloudWatchLogsDetails: CloudWatchLogsDetail | None = None
    KinesisFirehoseDetails: KinesisFirehoseDetail | None = None


from pydantic import BaseModel, StrictStr, Field, conlist
from typing import Literal
from .match_option_types import MatchOptionTypes


DimensionKeys = Literal[
    "AZ",
    "INSTANCE_TYPE",
    "LINKED_ACCOUNT",
    "LINKED_ACCOUNT_NAME",
    "OPERATION",
    "PURCHASE_TYPE",
    "REGION",
    "SERVICE",
    "SERVICE_CODE",
    "USAGE_TYPE",
    "USAGE_TYPE_GROUP",
    "RECORD_TYPE",
    "OPERATING_SYSTEM",
    "TENANCY",
    "SCOPE",
    "PLATFORM",
    "SUBSCRIPTION_ID",
    "LEGAL_ENTITY_NAME",
    "DEPLOYMENT_OPTION",
    "DATABASE_ENGINE",
    "CACHE_ENGINE",
    "INSTANCE_TYPE_FAMILY",
    "BILLING_ENTITY",
    "RESERVATION_ID",
    "RESOURCE_ID",
    "RIGHTSIZING_TYPE",
    "SAVINGS_PLANS_TYPE",
    "SAVINGS_PLAN_ARN",
    "PAYMENT_OPTION",
    "AGREEMENT_END_DATE_TIME_AFTER",
    "AGREEMENT_END_DATE_TIME_BEFORE",
    "INVOICING_ENTITY",
    "ANOMALY_TOTAL_IMPACT_ABSOLUTE",
    "ANOMALY_TOTAL_IMPACT_PERCENTAGE",
]


class Dimension(BaseModel):
    Key: DimensionKeys
    MatchOptions: list[MatchOptionTypes]
    Values: list[StrictStr]

    def dump(self):
        return self.model_dump()


import asyncio
import boto3
import functools
from botocore.config import Config
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.exceptions import UnsetAWSConnectionException
from werkflow_aws.models import (
    AWSCredentialsSet,
    AWSRegionMap,
    RegionName,
)
from werkflow_aws.models.elasticache import (
    DescribeCacheClustersRequest,
    DescribeCacheClustersResponse,
)

from werkflow_aws.types import ElastiCacheClient
from werkflow_system import System
from typing import Union


class AWSElastiCache:
    
    def __init__(self) -> None:
        
        self._system = System()

        self._loop: Union[asyncio.AbstractEventLoop, None] = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None

        self.service_name = 'ElastiCache'
        self._regions = AWSRegionMap()
    
        
    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            None,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

        self._client: ElastiCacheClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'elasticache',
            )
        )

    async def connect(
        self,
        credentials: AWSCredentialsSet,
        region: RegionName,
    ):
        
        aws_region = self._regions.get(region)

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._client: ElastiCacheClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'elasticache',
                aws_access_key_id=credentials.aws_access_key_id,
                aws_secret_access_key=credentials.aws_secret_access_key,
                aws_session_token=credentials.aws_session_token,
                config=Config(
                    region_name=aws_region.value
                )
            )
        )

    async def describe_cache_clusters(
        self,
        request: DescribeCacheClustersRequest
    ) -> DescribeCacheClustersResponse:
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name,
            )
        
        dumped = request.model_dump(exclude_none=True)
        
        result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.describe_cache_clusters(
                    CacheClusterId=dumped.get('CacheClusterId'),
                    MaxRecords=dumped.get('MaxRecords'),
                    Marker=dumped.get('Marker'),
                    ShowCacheNodeInfo=dumped.get('ShowCacheNodeInfo'),
                    ShowCacheClustersNotInReplicationGroups=dumped.get('ShowCacheClustersNotInReplicationGroups')
                ),
            )
        )

        return DescribeCacheClustersResponse(**result)
    
    async def close(self):

        await self._loop.run_in_executor(
            None,
            self._client.close
        )

        await self._system.close()
        self._executor.shutdown()

    def abort(self):
        self._client.close()
        self._system.abort()
        self._executor.shutdown()


from pydantic import BaseModel, StrictStr, StrictBool
from typing import Optional, Literal
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)


class AWSs3CompleteMultipartUploadResponse(BaseModel):
    Location: StrictStr
    Bucket: StrictStr
    Key: StrictStr
    Expiration: StrictStr
    ETag: StrictStr
    ChecksumCRC32: StrictStr
    ChecksumCRC32C: StrictStr
    ChecksumSHA1: StrictStr
    ChecksumSHA256: StrictStr
    ServerSideEncryption: Literal[
        'AES256',
        'aws:kms',
        'aws:kms:dsse'
    ]
    VersionId: StrictStr
    SSEKMSKeyId: Optional[StrictStr]
    BucketKeyEnabled: StrictBool
    RequestCharged: Optional[
        Literal['requester']
    ]=None


from pydantic import (
    BaseModel,
    StrictStr
)
from typing import List
from .aws_s3_tag import AWSs3Tag


class AWSs3GetObjectTaggingResponse(BaseModel):
    VersionId: StrictStr
    TagSet: List[AWSs3Tag]=[]


from pydantic import (
    StrictStr
)
from typing import Literal, Optional
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3PutBucketTaggingOptions(AWSBoto3Base):
    checksum_algorithm: Optional[
        Literal[
            'CRC32',
            'CRC32C',
            'SHA1',
            'SHA256'
        ]
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None


from pydantic import BaseModel, StrictStr, StrictBool
from typing import Literal, Optional


class AWSs3DeleteObjectResponse(BaseModel):
    DeleteMarker: StrictBool
    VersionId: StrictStr
    RequestCharged: Optional[
        Literal['requester']
    ]=None


from pydantic import StrictStr
from typing import Optional, Literal
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)


class AWSs3CompleteMultipartUploadOptions(AWSBoto3Base):
    checksum_crc32: Optional[StrictStr]=None
    checksum_crc32c: Optional[StrictStr]=None
    checksum_sha1: Optional[StrictStr]=None
    checksum_sha256: Optional[StrictStr]=None
    expected_bucket_owner: Optional[StrictStr]=None
    sse_customer_algorithm: Optional[StrictStr]=None
    sse_customer_key: Optional[StrictStr]=None
    request_payer: Optional[
        Literal['requester']
    ]=None
    upload_id: Optional[StrictStr]=None

    def to_options(self):

        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'crc32',
                    'crc32c',
                    'sha1',
                    'sha256',
                    'sse'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'content_md5',
                    'checksum_crc32',
                    'checksum_crc32c',
                    'checksum_sha1',
                    'checksum_sha256',
                    'grant_read_acp',
                    'grant_write_acp',
                    'sse_customer_algorithm',
                    'sse_customer_key'
                ]
            )
        }


        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items()
        }

        parsed_options.update(uppercased_options)

        return parsed_options



from werkflow_core import Module
from .parsers import AWSConfigParser
from .services import (
    AWSCredentials,
    AWSCodeArtifact,
    AWSCostExplorer,
    AWSElastiCache,
    AWSSTS,
)


class AWS(Module):

    def __init__(self) -> None:
        super().__init__()

        self.config = AWSConfigParser()
        self.credentials = AWSCredentials()
        self.code_artifact = AWSCodeArtifact()
        self.cost_explorer = AWSCostExplorer()
        self.elasticache = AWSElastiCache()
        self.sts = AWSSTS()

    async def close(self):
        await self.credentials.close()
        await self.code_artifact.close()
        await self.cost_explorer.close()
        await self.elasticache.close()

    def abort(self):
        self.credentials.abort()
        self.code_artifact.abort()
        self.cost_explorer.abort()
        self.elasticache.abort()

    




from pydantic import BaseModel, StrictStr
from typing import Dict, Any


class AWSs3CreateMultipartUploadResponse(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    UploadId: StrictStr
    ResponseMetadata: Dict[StrictStr, Any]


import datetime
from pydantic import BaseModel, StrictStr, StrictInt, StrictBool
from typing import Literal
from .configuration_endpoint import ConfigurationEndpoint
from .log_delivery_configuration import LogDeliveryConfiguration
from .pending_modified_values import PendingModifiedValues
from .notification_configuration import NotificationConfiguration
from .cache_secruity_group import CacheSecurityGroup
from .cache_parameter_group import CacheParameterGroup
from .cache_node import CacheNode
from .security_group import SecurityGroup


class CacheCluster(BaseModel):
    CacheClusterId: StrictStr
    ConfigurationEndpoint: ConfigurationEndpoint
    ClientDownloadLandingPage: StrictStr
    Engine: StrictStr
    EngineVersion: StrictStr
    CacheClusterStatus: StrictStr
    NumCacheNodes: StrictInt
    PreferredAvailabilityZone: StrictStr
    PreferredOutpostArn: StrictStr
    CacheClusterCreateTime: datetime.datetime
    PreferredMaintenanceWindow: StrictStr
    PendingModifiedValues: PendingModifiedValues
    NotificationConfiguration: NotificationConfiguration
    CacheSecurityGroups: list[CacheSecurityGroup]
    CacheParameterGroup: CacheParameterGroup
    CacheSubnetGroupName: StrictStr
    CacheNodes: list[CacheNode]
    AutoMinorVersionUpgrade: StrictBool
    SecurityGroups: list[SecurityGroup]
    ReplicationGroupId: StrictStr
    SnapshotRetentionLimit: StrictInt
    SnapshotWindow: StrictStr
    AuthTokenEnabled: StrictBool
    AuthTokenLastModifiedDate: datetime.datetime
    TransitEncryptionEnabled: StrictBool
    AtRestEncryptionEnabled: StrictBool
    ARN: StrictStr
    ReplicationGroupLogDeliveryEnabled: StrictBool
    LogDeliveryConfigurations: list[LogDeliveryConfiguration]
    NetworkType: Literal[
        'ipv4',
        'ipv6',
        'dual_stack',
    ]
    IpDiscovery: Literal[
        'ipv4',
        'ipv6',
    ]
    TransitEncryptionMode: Literal[
        'preferred',
        'required',
    ]


from .key_conversion import (
    convert_key_to_boto3_arg as convert_key_to_boto3_arg,
    convert_key_to_boto3_arg_upper_matching as convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns as key_contains_patterns
)
from .aws_config_role import AWSConfigRole as AWSConfigRole


from .aws import AWS as AWS
from .models import (
    AWSCredentialsSet as AWSCredentialsSet,
    AWSRegion as AWSRegion,
    AWSRegionMap as AWSRegionMap,
    RegionName as RegionName,
    AWSServices as AWSServices,
    AWSServicesMap as AWSServicesMap,
    ServiceName as ServiceName,
)


import datetime
from pydantic import (
    BaseModel,
    StrictStr
)
from typing import Dict, Literal


class AWSs3MultipartUploadResponse(BaseModel):
    UploadId: StrictStr
    Key: StrictStr
    Initiated: datetime.datetime
    StorageClass: Literal[
        'STANDARD',
        'REDUCED_REDUNDANCY',
        'STANDARD_IA',
        'ONEZONE_IA',
        'INTELLIGENT_TIERING',
        'GLACIER',
        'DEEP_ARCHIVE',
        'OUTPOSTS',
        'GLACIER_IR',
        'SNOW',
        'EXPRESS_ONEZONE'
    ]
    Owner: Dict[
        Literal['DisplayName', 'ID'],
        StrictStr
    ]
    Initiator: Dict[
        Literal[
            'ID',
            'DisplayName'
        ],
        StrictStr
    ]
    ChecksumAlgorithm: Literal[
        'CRC32',
        'CRC32C',
        'SHA1',
        'SHA256'
    ]


from pydantic import BaseModel, StrictStr, StrictInt, StrictBool


class DescribeCacheClustersRequest(BaseModel):
        CacheClusterId: StrictStr = None,
        MaxRecords: StrictInt = 100,
        Marker: StrictStr | None = None,
        ShowCacheNodeInfo: StrictBool = True,
        ShowCacheClustersNotInReplicationGroups: StrictBool = True,



from pydantic import BaseModel, StrictStr, StrictInt
from .credentials import Credentials
from .role_user import RoleUser


class AssumedRoleResponse(BaseModel):
    Credentials: Credentials
    AssumedRoleUser: RoleUser | None = None
    PackedPolicySize: StrictInt | None = None
    SourceIdentity: StrictStr | None = None
